//////////////////////////////////////////////////////////////////////////////
//   File name:  j_imaging.C
//   Version:    see below
//   Component:  j_imaging
//   Developer:  Peter Kretschmar, ISDC
//               Peter.Kretschmar@obs.unige.ch
//               Silvia Martnez Nez, GACE, Valencia University
//               Silvia.Martinez@uv.es
//               Simona Soldi, ISDC
//               Simona.Soldi@obs.unige.ch
//               Lucia Pavan, ISDC
//               Lucia.Pavan@unige.ch
//
//   Purpose:
//
//   Ties together j_ima_iros and q_identify_srcs. j_ima_iros takes the existing 
//   shadowgrams, deconvolving the images, searching for sources and estimating their fluxes.
//   q_identify_srcs identifies the found sources as known or as new by comparing their found
//   positions with catalogue sources positions.
//
//
//   History:
//   1.0 - 2000-12-06 - Version used at Dec 2000 Consortium Meeting demo.
//   2.0 - 2001-06-15 - Script structure largely rewritten to make use of 
//                      improved ISDCTask() capabilities.
//   3.0 - 2001-09-12 - Updated script with isdcroot 3.1 improvements     
//   3.1 - 2001-11-12 - Common high-level script parameters
//   3.2 - 2002-05-09 - Added parameters for called executables [SMN]
//   3.3 - 2002-06-14 - A new function is defined jemx_check_shd_idx 
//                      to check if imaging analysis is possible [SMN]
//   3.4 - 2002-07-05 - Use DALparseDOL [PK]
//   3.5 - 2002-12-27 - Allow for direct instMod parameter [PK]
//   3.6 - 2003-02-13 - Run q_identify_srcs only if we have JMXi-SRCL-CAT [SMN]
//   3.7.0 - 2003-06-19 - Give all scripts same version [PK] 
//   3.7.4 - 2003-10-03 - SCREWs 1106 -limit source finding to user-defined 
//                        angle (new parameter angleLimit- and 1230 - 
//                        use coordinates from user catalog on request 
//                        (new parmater useCatPos)-. [SMN]
//   4.2.0 - 2004-05-12 - SCREW 1454: include j_ima_fine_resol
//   4.3.0 - 2004-09-06 - Allow for different subdir structure (SCREW 1532) [PK]
//   4.4.1 - 2004-12-01 - Correct parameter numPixelSubDiv (SPR 3939) [PK]
//   4.5.0 - 2005-04-12 - Major changes to include j_ima_iros instead of 
//                       the old imaging software [SMN]
//   4.5.1 - 2005-05-19 - IMA_dolBPL parameter has to be set by the script
//                        (SPR 4153) [SMN]
//   4.6   - 2006-09-12 - Updated ISDCTask Setting SPR-04571 [SMN]
//   5.0   - 2007-07-18 - Change of the parameter interactionDepth default 
//			  value (SCREW 2001) [So]
//    			- Include parameter detImagesOut with the option to delete 
//			  the shadowgrams and images used for the source detection
//			  (SCREW 2001) [So]
//   6.0.0 - 2008-11-13 - Change of units for the parameter fluxLimit (SPR 4728) [So] 
//    			- pixelFold parameter added (SPR 4820) [So]
//                      - Eliminate from the j_scripts the option to delete 
//                        the shadowgrams and images used for the source detection.
//                        Now j_ima_iros does it (SCREW 2098) [So]
//			- Introduced the new parameters userImagesOut, useTrace
//                        and tracestring and modified the parameter detImagesOut
//                        as a j_ima_iros parameter (SCREW 2098) [So]
//  6.2  -2016-02-24  [LP] unpdate of parameter files to handle the new j_ima_iros (with LC extraction included)
//
//////////////////////////////////////////////////////////////////////////////

// Standard setup
#define ISDCMAIN j_imaging
#define MAIN_VER "6.2.1"
#include "scriptstuff.h"

// Instrument-specific error codes defined by JEM-X SDAST
#define J_IMA_SCRIPT_ERROR_START   (J_ERROR_CODE_START - 32900)
#define J_IMA_IC_DATA_NOT_FOUND    (J_IMA_SCRIPT_ERROR_START - 1)  

// Internal specific error 
#define J_IMA_NO_SUCH_DATA         (J_IMA_SCRIPT_ERROR_START - 99)
#define J_IMA_NO_EVTS_SHD_IDX      (J_IMA_SCRIPT_ERROR_START - 98)
 
// Verbosity levels
#define J_CHATTY_VERY_QUIET   0 /* Only system messages and fatal errors */
#define J_CHATTY_QUIET        1 /* Errors and severe warnings only */
#define J_CHATTY_NORMAL       2 /* + important Log messages, extend warnings */
#define J_CHATTY_VERBOSE      3 /* + program flow comments */
#define J_CHATTY_VERY_VERBOSE 4 /* + miscellaneous program control values */
#define J_CHATTY_MAXIMAL      5 /* Everything you want */


using namespace std;
using namespace ISDC;


// Function prototypes for script functions

int jemx_reportTaskStatus(ISDCTask *task, int chatter);

int jima_SetUp(const char  *SWG,
	       const char  *icGroup,
	       const char  *icAlias,
	       int         jemxNum,
	       char        *relPath,
	       char        *instMod,
	       char        *listCat,
	       char        *dolBPL,
	       int         chatter,
	       int         callingStatus);

int jemx_check_evts_shd_idx(int         jemxNum,
                            const char  *SWG,
                            int         chatter,
                            int         callingStatus);

//*************************************************************************
// Now to the actual script which includes the following steps:
// - Set up script as ISDCTask 
// - Set up called executables as ISDCTasks 
// - Set up some variables used in the further processing
// - Check if imaging data are available, if not, the executables will not be ran
// - Run j_ima_iros        to create deconvolved images
//                         to find sources in the images
//                         to correct the obtained source fluxes
//                         for vignetting effects.
// - Run q_identify_srcs   to identify the sources.
//*************************************************************************

	


int j_imaging(int argc, char ** argv)
{
    // Use standard namespaces
    using namespace std;
    using namespace ISDC;


    // Local variable definitions
    int     status = ISDC_OK;                 // Processing status 
    int     jemxNum=0;                        // JEM-X detector number
    int     chatter=0;                        // Verbosity level

    char    rilString[DAL_MAX_STRING];        // Used in RIL calls
    char    instMod[DAL_FILE_NAME_STRING];    // DOL for instrument model
    char    relPath[DAL_FILE_NAME_STRING];    // Relative path to SWG    
    char    subDir[DAL_FILE_NAME_STRING];     // Sub dir. below SWG for results
    char    skyRes[DAL_FILE_NAME_STRING];     // DOL for images in sky coord.
    char    listCat[DAL_FILE_NAME_STRING];    // DOL for expected source list
    char    listRes[DAL_FILE_NAME_STRING];    // DOL for source list
//////LP
    char    lcnewRes[DAL_FILE_NAME_STRING];    // DOL for lc file (new software 2016)
//////LP

    char    dolBPL[DAL_FILE_NAME_STRING];     // DOL for JMXi BPL (backprojection list) group 

    // one-pass do loop for error control
    do {

        // 
	// ==== Set up script ===============================================
	//
	// The script itself is set up as ISDCTask object. This ensures
	// correct setup of some ISDC library functions and simplifies
        // access to the script parameters within the script itself.
	//

	ISDCTask j_imaging(ISDCMAINNAME,
			   MAIN_VER,
			   argc,argv,
			   ISDCTask::kPARAMGUI |ISDCTask::kCLI);
	status = j_imaging.GetStatus();
	
	if (status==kUSER_ABORT){
	    sprintf(rilString,"Quitting upon User Request.");
	    RILlogMessage(NULL,Warning_0,rilString);
	    break;
	    }

	if (ISDC_OK != status) { 
	    sprintf(rilString,
		    "Could not set up script as ISDCTask! status=%d",
		    status);
	    RILlogMessage(NULL,Error_2,rilString);
	    break; 
	    }
	 
	chatter    = static_cast<int>(j_imaging["chatter"]);
	jemxNum   = static_cast<int>(j_imaging["jemxNum"]);
	


       // 
       // ==== Call jemx_check_evts_shd_idx  ========================
       //
       // This function checks if we have imaging data, checking if the index
       // table JMXi-EVTS-SHD-IDX contains shadowgrams or no. The previous
       // level (BIN_I) will produce shadowgrams only if imaging data are 
       // available.
       //	
	
        status=jemx_check_evts_shd_idx(jemxNum,(const char*)j_imaging["swgDOL"],chatter,status);
	
       if (ISDC_OK!=status && J_IMA_NO_SUCH_DATA!=status && J_IMA_NO_EVTS_SHD_IDX!=status) { 
	    sprintf(rilString,"jemx_check_evts_shd_idx(): Error %d while checking index"
		    "JMX%1d-EVTS-SHD-IDX",status,jemxNum);
	    RILlogMessage(NULL,Error_2,rilString);
	    break;}

       //If imaging data are not available (the index table is empty or does not exists), the
       //script will stop this level without running the executables.

       if (J_IMA_NO_EVTS_SHD_IDX==status){
	  sprintf(rilString,
 		   "The index table JMX%1d-EVTS-SHD does not exists on SWG %s."
                   "Imaging data are not available and then imaging analysis is not" 
                   " possible.",
 		    jemxNum,(const char*)j_imaging["swgDOL"]);
 	   RILlogString(NULL,Warning_2,rilString);
	   status=ISDC_OK;
           break;}  
       	   
       if (J_IMA_NO_SUCH_DATA==status){
	    sprintf(rilString,
 		   "The index table JMX%1d-EVTS-SHD is empty on SWG %s"
                   "Imaging data are not available and then imaging analysis is not" 
                   " possible.",
 		   jemxNum,(const char*)j_imaging["swgDOL"]);
 	   RILlogString(NULL,Warning_2,rilString);
	   status=ISDC_OK;
           break;} 
        

	//
	// ==== Set up called executables ==============================
	//
	// The executables are set up as ISDCTask objects.
	// If anything goes wrong with this, we break to the end.
	
	ISDCTask j_ima_iros("j_ima_iros");
	status = jemx_reportTaskStatus(&j_ima_iros,chatter);
	if (ISDC_OK != status) break;

	ISDCTask q_identify_srcs("q_identify_srcs");
	status = jemx_reportTaskStatus(&q_identify_srcs,chatter);
	if (ISDC_OK != status) break;

	//
	// ==== Set up generic variables ================================
	// For readability, the commands to set up some variables used in
        // the following have been combined in a single function. 
	// Among other things, this function accesses the IC database 
        // for the address of the JEM-X instrument model group.
	//

        strcpy(instMod,(const char *)j_imaging["instMod"]);
	strcpy(subDir,(const char *)j_imaging["subDir"]);
        strcpy(dolBPL,(const char *)j_imaging["dolBPL"]);

	status = jima_SetUp((const char *)j_imaging["swgDOL"],
			    (const char *)j_imaging["IC_Group"],
			    (const char *)j_imaging["IC_Alias"],
			    jemxNum,
			    relPath,  // for output DOLs
			    instMod,  // DOL of JEM-X instr. model group
			    listCat,  // DOL of JMXi-SRCL-CAT
			    dolBPL,  //  DOL of JMXi-BPL-GRP
			    chatter,
			    status);

	if(ISDC_OK != status) {
	    sprintf(rilString,"Could not set up script variables! "
		    "status = %d",status);
	    RILlogMessage(NULL,Error_2,rilString);
	    break;
	}

	//
	// ==== Call j_ima_iros ==================================
	//
	// This executable creates a deconvolved image for each shadowgram
	// present in the Science Window Group, look for the sources and
	// determinate the fluxes.
	
	// Set up DOLs for outputs


	sprintf(skyRes,
		"%s/%s/jmx%1d_sky_ima.fits(JMX%1d-SKY.-IMA-IDX.tpl)",
		relPath,subDir,jemxNum,jemxNum);
	sprintf(listRes,
		"%s/%s/jmx%1d_srcl_res.fits(JMX%1d-SRCL-RES.tpl)",
		relPath,subDir,jemxNum,jemxNum);
////// LP
	sprintf(lcnewRes,
		"%s/%s/jmx%1d_src_iros_lc.fits(JMX%1d-SRC.-LCR-IDX.tpl)",
		relPath,subDir,jemxNum,jemxNum);
////// LP



	j_ima_iros["inSWG"]       = "";
	j_ima_iros["outSWG"]      = j_imaging["swgDOL"];
	j_ima_iros["instMod"]     = instMod;
	j_ima_iros["skyRes"]      = skyRes;
	j_ima_iros["listRes"]     = listRes;

///// LP

	j_ima_iros["lcnewRes"]     = lcnewRes;
/////
	j_ima_iros["jemxNum"]     = jemxNum;
	j_ima_iros["chatter"]     = chatter;
        j_ima_iros["makeNewBPL"]        = j_imaging["makeNewBPL"];
	j_ima_iros["newBackProjFile"]   = j_imaging["newBackProjFile"];
	j_ima_iros["detAccLimit"]       = j_imaging["detAccLimit"];
	j_ima_iros["skyImageDim"]       = j_imaging["skyImageDim"];
	j_ima_iros["useDeadAnodes"]     = j_imaging["useDeadAnodes"];
	j_ima_iros["maxNumSources"]     = j_imaging["maxNumSources"];
	j_ima_iros["edgeEnhanceFactor"] = j_imaging["edgeEnhanceFactor"];
	j_ima_iros["loopLimitPeak"]     = j_imaging["loopLimitPeak"];
	j_ima_iros["detSigSingle"]      = j_imaging["detSigSingle"];
	j_ima_iros["pixelFold"]         = j_imaging["pixelFold"];
	j_ima_iros["skyRadiusFactor"]   = j_imaging["skyRadiusFactor"];	
	j_ima_iros["radiusLimit0"]      = j_imaging["radiusLimit0"];
	j_ima_iros["radiusLimit1"]      = j_imaging["radiusLimit1"];
	j_ima_iros["radiusLimit2"]      = j_imaging["radiusLimit2"];
	j_ima_iros["radiusLimit3"]      = j_imaging["radiusLimit3"];
	j_ima_iros["interactionDepth"]  = j_imaging["interactionDepth"];
	j_ima_iros["hotPixelLimit"]     = j_imaging["hotPixelLimit"];
	j_ima_iros["skyImagesOut"]      = j_imaging["skyImagesOut"];
	j_ima_iros["dolBPL"]            = dolBPL;
	j_ima_iros["bkgShdDOL"]         = j_imaging["bkgShdDOL"];
	j_ima_iros["detImagesOut"]      = j_imaging["detImagesOut"];
	j_ima_iros["userImagesOut"]     = j_imaging["userImagesOut"];
	j_ima_iros["useTrace"]          = j_imaging["useTrace"];
	j_ima_iros["tracestring"]       = j_imaging["tracestring"];
	j_ima_iros["makeLCs"]	        = j_imaging["makeLCs"];
	j_ima_iros["timeStep"]          = j_imaging["timeStep"];
	j_ima_iros["doBurstSearch"]     = j_imaging["doBurstSearch"];
	j_ima_iros["referenceTime"]     = j_imaging["referenceTime"];
	j_ima_iros["burstThreshSrc"]    = j_imaging["burstThreshSrc"];
	j_ima_iros["burstThreshDet"]    = j_imaging["burstThreshDet"];


	status = j_ima_iros.Run();
	if (ISDC_OK != status) {
	    sprintf(rilString,"Error deconvolving images ! status = %d",
		    status);
	    RILlogMessage(NULL,Error_2,rilString);
	    break;
	}


	//
	// ==== Call q_identify_srcs =====================================
	//
	// Since the ISSW does not identify the sources on its own,
	// call a generic tool to do so. The tool will be set up only if
	// JMXi-SRCL-CAT exists (then listCat is not an empty string)
	// 
	//
	
		if(strlen(listCat)!=0){

		sprintf(listRes,
			"%s/%s/jmx%1d_srcl_res.fits[1]",
		relPath,subDir,jemxNum,jemxNum);
		q_identify_srcs["srcl_cat_dol"] = listCat;
	        q_identify_srcs["srcl_res_dol"] = listRes;
		q_identify_srcs["instrument"]   = jemxNum+3;
		q_identify_srcs["unit"]         = 1;
		q_identify_srcs["alert_new"]    = 4;
		q_identify_srcs["alert_lost"]   = 4;
	        q_identify_srcs["alert_conf"]   = 4;
		q_identify_srcs["alert_mult"]   = 4;
		q_identify_srcs["alert_shift"]  = 4;
	        q_identify_srcs["chat"]         = (int)pow((float)10.0,chatter-3);
		q_identify_srcs["relDist"]      = j_imaging["relDist"];
		q_identify_srcs["fluxLimit"]    = j_imaging["fluxLimit"];
	        q_identify_srcs["searchRad"]    = j_imaging["searchRad"];
		q_identify_srcs["gridNum"]      = j_imaging["gridNum"];
		q_identify_srcs["distFuzz"]     = j_imaging["distFuzz"];

		status = q_identify_srcs.Run();
		if (ISDC_OK != status) {
		    sprintf(rilString,"Error identifying sources! status = %d",
			    status);
		    RILlogMessage(NULL,Error_2,rilString);
		    break;
		}	
		}else{

		    sprintf(rilString,"Tool for identifying sources not run."
			    "None JMXi-SRCL-CAT available");
		    RILlogMessage(NULL,Warning_2,rilString);
		    break;
		}
	
    } while (0); 
    // end of single-pass do loop
    
    return(status);
}

/***************************************************************************

  Function: jima_SetUp

  Description:
      
    Sets up several variables for use in the script. 
    These commands were grouped in a function for better visibility of
    the main script.

  Error codes:

     0 == ISDC_OK   ==> function executed with no errors
     < 0 (various)  ==> ISDC system function error code, passed on

  Parameter arguments:
     name           range    I/O     description

     SWG            ---      input   DOL to the Science WindowGroup
     icGroup        ---      input   DOL of the IC Master Group
     icAlias        ---      input   Alias for IC configuration selection
     jemxNum        1 or 2   input   JEM-X detector number
     relPath        ---      output  relative path to SWG used for output DS
     instMod        ---      output  DOL of the JEM-X instr. model group
     dolBPL         ---      output  DOL of the JEM-X backprojection list group
     chatter        0 to 5   input   Verbosity level
     callingStatus  any      input   Error status on input

******************************************************************************/

int jima_SetUp(const char  *SWG,
	       const char  *icGroup,
	       const char  *icAlias,
	       int         jemxNum,
	       char        *relPath,
	       char        *instMod,
	       char        *listCat,
	       char        *dolBPL,
	       int         chatter,
	       int         callingStatus)
{
    // local variables
    int          status=ISDC_OK;               // Local status
    int          pathLen=0;                    // Used in parsing path to SWG
    double       swgStartTime=0.0;             // Start time of SWG
    char         rilString[DAL_MAX_STRING];    // Used in RIL calls
    char         catName[DAL_MAX_STRING];      // Holds name of SRCL-CAT
    char         icName[DAL_MAX_STRING];       // Holds name of IC DS
    char        *strPtr=NULL;                  // Used to split strings
    char         filePath[DAL_FILE_NAME_STRING]; // Holds filename + path
    dal_element *dalPtr=NULL;                  // Ptr to Science Window Group
    dal_element *catPtr=NULL;                  // Ptr to Science Window Group
    int         flag_no_srcl_cat=0;            // flag_no_srcl_cat=1 if we do not have JMXi-SRCL-CAT 
                           
    // Exit immediately if calling status is not OK
    if (ISDC_OK != callingStatus) { return(callingStatus); }

   
    // Set relative path based on SWG
    status = DALparseDOL(SWG,NULL,NULL,NULL,NULL,relPath,
			 NULL,NULL,NULL,NULL,NULL,status);
    if (ISDC_OK != status) {
	sprintf(rilString,
		"jima_SetUp(): Could not parse SWG DOL %s! status=%d",
		SWG,status);
	RILlogMessage(NULL,Error_2,rilString);
	return(status);
    }

    // Retrieve the start time of the Science Window
    status = DALobjectOpen(SWG,&dalPtr,status);
    if (ISDC_OK != status) {
	sprintf(rilString,
		"jima_SetUp(): Could not open SWG %s! status=%d",
		SWG,status);
	RILlogMessage(NULL,Error_2,rilString);
	return(status);
    }

    status = DALattributeGet(dalPtr,"TSTART",DAL_DOUBLE,
			     (void*)&swgStartTime,NULL,NULL,status);
    if (ISDC_OK != status) {
	sprintf(rilString,
		"jima_SetUp(): Could not get start time of ScW! status=%d",
		status);
	RILlogMessage(NULL,Error_2,rilString);
	return(status);
    }

    sprintf(catName,"JMX%1d-",jemxNum);
    strcpy(catName+5,"SRCL-CAT");

    status = DALobjectFindElement(dalPtr,catName,&catPtr,status);
    if (ISDC_OK==status){	
	status = DALelementGetLocation(catPtr,listCat,status);
	if (ISDC_OK!= status) {
	    sprintf(rilString,
		    "jima_SetUp(): Could not get location of %s! status=%d",
		    catName,status);
	    RILlogMessage(NULL,Error_2,rilString);
	    return(status);
	}
    }else if(DAL_ELEMENT_NOT_FOUND==status){
	sprintf(listCat,""); 
	status=ISDC_OK;
    }else{
	sprintf(rilString,"jima_SetUp(): Could not find the element %s !"
		"status=%d",catName,status);
    }

    status = DALobjectClose(dalPtr,DAL_SAVE,status);

    // Check instMod string.
    // If it is empty or not a valid DOL, 
    // access IC database for Instrument Model Group

    if (0<strlen(instMod)) {

	dalPtr=NULL;
	status = DALobjectOpen(instMod,&dalPtr,status);
	if (ISDC_OK != status) {
	    sprintf(rilString,
		    "jbint_SetUp(): "
		    "Could not open instrument model group %s! status=%d",
		    instMod,status);
	    RILlogString(NULL,Warning_2,rilString);
	    RILlogString(NULL,Warning_2,
			 "Using default instrument model from IC instead!");
	    status=ISDC_OK;
	    strcpy(instMod,"");
	} else {
	    status = DALobjectClose(dalPtr,DAL_SAVE,status);
	}
    }else{

	sprintf(icName,"JMX%1d-IMOD-GRP",jemxNum);

	if (J_CHATTY_VERBOSE <= chatter) {
	    sprintf(rilString,
		    "Searching for Instrument Model (%s) in IC database",
		    icName);
	    RILlogMessage(NULL,Log_1,rilString);
	}

	status = ICgetNewestDOL(icGroup,
				icAlias,
				icName,"",swgStartTime,
				instMod,status);

	if(ISDC_OK != status) {
	    sprintf(rilString,"ICgetNewestDOL failed for %s! status = %d",
		    icName,status);
	    RILlogMessage(NULL,Error_2,rilString);
	    status = J_IMA_IC_DATA_NOT_FOUND;
	    return(status);
	}


    }

     // Check dolBPL string.
     // If it is empty or not a valid DOL, 
     // access IC database for Instrument Model Group

    if (0<strlen(dolBPL)) {

	dalPtr=NULL;
	status = DALobjectOpen(dolBPL,&dalPtr,status);
	if (ISDC_OK != status) {
	    sprintf(rilString,
		    "jbint_SetUp(): "
		    "Could not open backprojection list group %s! status=%d",
		    dolBPL,status);
	    RILlogString(NULL,Warning_2,rilString);
	    RILlogString(NULL,Warning_2,
			 "Using default backprojection list group from IC instead!");
	    status=ISDC_OK;
	    strcpy(dolBPL,"");
	} else {
	    status = DALobjectClose(dalPtr,DAL_SAVE,status);
	}
    }else{

	sprintf(icName,"JMX%1d-BPL.-GRP",jemxNum);

	if (J_CHATTY_VERBOSE <= chatter) {
	    sprintf(rilString,
		    "Searching for backprojection list group (%s) in IC database:",
		    icName);
	    RILlogMessage(NULL,Log_1,rilString);
	}
	status = ICgetNewestDOL(icGroup,
				icAlias,
				icName,"",swgStartTime,
				dolBPL,status);
	if(ISDC_OK != status) {
	    sprintf(rilString,"ICgetNewestDOL failed for %s! in %s status = %d",
		    icName,icGroup,status);
	    RILlogMessage(NULL,Error_2,rilString);
	    status = J_IMA_IC_DATA_NOT_FOUND;
	    return(status);
	}

    }

    return(status);
}


//******************************************************************
//
// Function jemx_reportTaskStatus
//
//******************************************************************

int jemx_reportTaskStatus(ISDCTask *task, int chatter) 
{
    using namespace std;
    using namespace ISDC;
    int   status=ISDC_OK;
    char  rilString[DAL_MAX_STRING];

    status = task->GetStatus();
    if (ISDC_OK!=status) {
	sprintf(rilString,"Problem with task - its status is %d!",
		status);
//		task.TaskName(),status);
	RILlogString(NULL,Error_2,rilString);
    } else if (J_CHATTY_VERBOSE<=chatter) {
	sprintf(rilString,"Task %s set up without problems",
		task->TaskName());
	RILlogString(NULL,Log_0,rilString);
    }

    return(status);
}


/***************************************************************************

  Function: jemx_check_evts_shd_idx

  Description:
      
    Checks the number of children of JMXi-EVTS-SHD-IDX contains, if number of children
    = 0 or the index table does not exists then the data for imaging are not available 
    and we need to skip the imaging analysis step.

  Error codes:

     0 == ISDC_OK   ==> function executed with no errors
     < 0 (various)  ==> ISDC system function error code, passed on

  Parameter arguments:
     name           range    I/O     description

     SWG            ---      input   DOL to the Science WindowGroup
     jemxNum        1 or 2   input   JEM-X detector number
     evtType        -1,0,1   input   Type of event (0=FULL,1=REST,-1=BOTH)
     chatter        0 to 5   input   Verbosity level
     callingStatus  any      input   Error status on input

******************************************************************************/


int jemx_check_evts_shd_idx(int         jemxNum,
			    const char  *SWG,
			    int         chatter,
			    int         callingStatus){

// Local variables

    int          status=ISDC_OK;                // Local status
    dal_element *swgPtr=NULL;                   // Ptr to Science Window Group
    char         rilString[DAL_MAX_STRING];     // Used in RIL calls
    char         index_name[DAL_MAX_STRING];    // JMXi-EVTS-SHD
    int          number_members=0;              // Children number of JMXi-EVTS-SHD-IDX
    int          flag_index_table=1;            // Exists index table JMXi-EVTS-SHD-IDX (index_table=1)
   

// Exit immediately if calling status is not OK

    if (ISDC_OK != callingStatus) {return(callingStatus);} 

    do{ //Error handling loop

     sprintf(index_name,"JMX%1d-EVTS-SHD",jemxNum);

     status=DALobjectOpen(SWG,&swgPtr,status);
     if(ISDC_OK!=status){
	   sprintf(rilString,"jemx_check_evts_shd_idx():Could not open SWG %s!" 
		   "status= %d",SWG,status);
           RILlogMessage(NULL,Error_2,rilString);
           return(status);
         }
     status=DAL3GENindexGetNumMembers(swgPtr,index_name,&number_members,status);
    
     if(ISDC_OK!=status && DAL3GEN_INDEX_DOESNT_MATCH!=status){
	   sprintf(rilString,"jemx_check_evts_shd_idx():Could not check if the index" 
                   "table %s-IDX exists or contains data on SWG %s! status= %d",
		   index_name,SWG,status);
           RILlogMessage(NULL,Warning_2,rilString);
           return(status);
     }
   
     if (DAL3GEN_INDEX_DOESNT_MATCH==status){
	 flag_index_table=0;   //JMXi-EVTS-SHD-IDX does not exists !
	 status=ISDC_OK;
     }

	
     if(J_CHATTY_VERBOSE<=chatter){
	 sprintf(rilString,"Number of children that %s-IDX contains are: %d",
		 index_name,number_members);  
         RILlogMessage(NULL,Log_1,rilString);
       }

     status=DALobjectClose(swgPtr,DAL_SAVE,status);
     if(ISDC_OK!=status){
	 sprintf(rilString,"jemx_check_evts_shd_idx():Could not close SWG %s!"
		 "status= %d",SWG,status);
           RILlogMessage(NULL,Error_2,rilString);
           return(status);
         }

     if (number_members==0){status=J_IMA_NO_SUCH_DATA;}  
     if (flag_index_table==0){status=J_IMA_NO_EVTS_SHD_IDX;}
 	

 } while(0); //end of error handling loop

 return status;

}
